import os
import logging.config
from pathlib import Path
from functools import wraps

import yaml

def contendo_logging_setup(
    default_path='{}/contendo_logging.yaml'.format(Path(__file__).parent),
    default_level=logging.INFO,
    env_key='LOG_CFG'
):
    """Setup logging configuration

    """
    def set_config_level(config, level):
        map = {logging.DEBUG: 'DEBUG', logging.INFO: 'INFO', logging.WARNING: 'WARNING', logging.ERROR: 'ERROR', logging.FATAL: 'FATAL'}
        for key, value in config.items():
            if key=='level':
                config[key] = map[level]
            elif type(value)==dict:
                set_config_level(value, level)
            else:
                continue

    path = default_path
    value = os.getenv(env_key, None)
    if value:
        path = value
    if os.path.exists(path):
        with open(path, 'rt') as f:
            config = yaml.safe_load(f.read())
        set_config_level(config, default_level)
        logging.config.dictConfig(config)
    else:
        logging.basicConfig(level=default_level)


def contendo_function_logger(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        func.__globals__['logger'] = logging.getLogger(func.__name__)
        return func(*args, **kwargs)
    return wrapper

def contendo_classfunction_logger(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        func.__globals__['logger'] = logging.getLogger('{}.{}'.format(type(self).__name__, func.__name__))
        return func(self, *args, **kwargs)
    return wrapper

if __name__ == '__main__':
    contendo_logging_setup(default_level=logging.WARNING)



class SimpleTextInteraction:
    def __init__(self):
        pass

    ## Text menu in Python
    def get_selection(self, options: list, title = 'Please select', default = 1):  ## Your menu design here

        padding = 20
        while True:  ## While loop which will keep going until loop = False
            print(padding * "-", title, padding * "-")
            for i in range(len(options)):
                print("{}. {}".format(i + 1, options[i]))
            print('0. Exit')
            print((padding*2 + 2 + len(title)) * "-")
            choice = input("Enter your choice [0-{}]: ".format(len(options)))
            try:
                if choice == '':
                    return default
                elif int(choice) <= len(options):
                    return int(choice)
                else:
                    pass
            except:
                pass
            print("Wrong input. please try again...")

    def get_numeric(self, text, low=None, high=None):
        while True:
            try:
                val = input(text)
                val = int(val)
                if low is not None and val<low:
                    print('{} is smaller than {}, please enter again'.format(val,low))
                    continue
                if high is not None and val > high:
                    print('{} is bigger than {}, please enter again'.format(val, high))
                    continue
                return val
            except:
                print('"{}" is not numeric, please try again'.format(val))

if __name__ == '__main__':
    menu = SimpleTextInteraction()
    selection=1
    while selection != 0:
        selection = menu.get_selection(['Option 1', 'Option 2', 'Option 3'], default=2)
        print ('Selected:', selection)
        menu.get_numeric('Please enter a number between 3 and 8: ', 3, 8)




def print_dict(d: dict, indent = ''):
    print(dict_to_string(d))

def dict_to_string(d: dict, indent = ''):
    ret = ''
    for key, value in d.items():
        if type(value) == dict:
            ret += '{}{}:\n'.format(indent, key)
            ret += dict_to_string(value, indent+'  ')
        else:
            ret += '{}{}: {}\n'.format(indent, key, value)
    return ret

if __name__ == '__main__':
    d = {
        'a': 1,
        'sub': {
            'c': 3,
            'd': 4,
        },
        'b': 2,
    }
    print_dict(d)
    print(dict_to_string(d))